### XA1011 error when using D8 with ProGuard

Any project using the D8 DEX compiler that has either **Code shrinker**
(`$(AndroidLinkTool)`) set to **ProGuard** or the older
`$(AndroidEnableProguard)` MSBuild property set to `true` will now get a XA1011
build error:

```
error XA1011: Using ProGuard with the D8 DEX compiler is no longer supported. Please update `$(AndroidLinkTool)` to `r8`.
```

In the past this configuration was allowed, but now only the R8 code shrinker is
supported for use with the D8 DEX compiler.

To resolve this error, update the **Code shrinker** setting in the Visual Studio
project property pages to **r8**.  This corresponds to the `r8` value for the
`$(AndroidLinkTool)` MSBuild property in the *.csproj* file:

```xml
<PropertyGroup>
  <AndroidLinkTool>r8</AndroidLinkTool>
</PropertyGroup>
```

> [!NOTE]
> R8 might in some cases produce new build warnings or errors compared to
> ProGuard, so the ProGuard rules for the project might require a few updates to
> resolve any new warnings or errors that appear after this change.
